<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria; 
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\DespesasAdministrativasService;

class DespesasAdministrativasController extends Crud {

    protected $dataSourceName = 'getDespesasAdministrativas';
    protected $dataSourceName_Set = 'fpa_despesasAdministrativas_set';

    protected $parameterBagColumns = array('P_NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, DespesasAdministrativasService $despesasAdministrativasService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->despesasAdministrativasService = $despesasAdministrativasService;
        parent::__construct($dataSourceManager);
    }

    public function getDespesasAdministrativas (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setDespesasAdministrativas (DTO\Request\DataSet $request, DTO\Response $response) {
        $existPrimaryKey = 0;
        foreach($request->getDataSet()->getRows() as &$row) {
            if($row['__is_new']){
                $existPrimaryKey = $this->despesasAdministrativasService->validaRestricaoChavePrimaria($row['CDDESPADMFAT'], $row["NRORG"]);
                if(is_numeric($row['CDDESPADMFAT']) && $row['CDDESPADMFAT'] <= 99)
                    $row['CDDESPADMFAT'] = str_pad($row['CDDESPADMFAT'], 2, "0", STR_PAD_LEFT);
            }
            $this->dataSourceName = $this->dataSourceName_Set;
        }
        if($existPrimaryKey)
            $response->addNotification(new DTO\Response\Notification("HCM01062_FPA_DUPLICATE_PRIMARY_KEY_ERROR", DTO\Response\Notification::TYPE_ERROR));
        else 
            parent::save($request, $response);
    }
 
    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceName_Set;
        $existInEventoCtbl;
        foreach($request->getDataSet()->getRows() as &$row){
            $existInEventoCtbl = $this->despesasAdministrativasService->existenciaDespesaEmTabEventoCtbl($row['CDDESPADMFAT'], $row["NRORG"]);
        }
        if($existInEventoCtbl == 0)
            parent::delete($request, $response);
        else
            $response->addNotification(new DTO\Response\Notification("HCM01062_FPA_DESPESA_EXISTENTE_EM_EVENTCTBL", DTO\Response\Notification::TYPE_ERROR));
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}