<?php
namespace Fiscal\Logic\NotaFiscal;

use \Fiscal\Entities\Notafiscal;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use Geral\Util\DateUtil;
/**
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 25/10/12
 * Time: 08:33
 * To change this template use File | Settings | File Templates.
 */
class ProceduresNF
{

    /**
     * Executa a procedure GERA_NF_AF_PARCIAL, usada para gerar nota para AFs baixadas parcialmente.
     *
     * @static
     * @param Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function callProcedureBaixaParcial(Notafiscal $notaFiscal)
    {
        try {
            $procedure = new Procedure("GERA_NF_AF_PARCIAL");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $notaFiscal->getFilial()->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $notaFiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function callProcedureBaixaAF($paramsBaixaAF)
    {
        try {
            $procedure = new Procedure("NF_BAIXA_AF");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $paramsBaixaAF['CDFILIAL'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NRLANCTONF'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRSEQITPROD",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NRSEQITPROD'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_IDSTATUS",
                Param::PARAM_INPUT,
                $paramsBaixaAF['IDSTATUS'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NEWCDPRODEMIS",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NEWCDPRODEMIS'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_OLDCDPRODEMIS",
                Param::PARAM_INPUT,
                $paramsBaixaAF['OLDCDPRODEMIS'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NEWCDPRODUTO",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NEWCDPRODUTO'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_OLDCDPRODUTO",
                Param::PARAM_INPUT,
                $paramsBaixaAF['OLDCDPRODUTO'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NEWCDMARCA",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NEWCDMARCA'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_OLDCDMARCA",
                Param::PARAM_INPUT,
                $paramsBaixaAF['OLDCDMARCA'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NEWQTITEMPROD",
                Param::PARAM_INPUT,
                $paramsBaixaAF['NEWQTITEMPROD'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_OLDQTITEMPROD",
                Param::PARAM_INPUT,
                $paramsBaixaAF['OLDQTITEMPROD'],
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_IDERRO",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_STR,
                1
            ));
            $erro = $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Chama a procedure CONSULTA_GRUPOFISCAL para verificar se existe algum grupo fiscal na parametrizacao.
     *
     * @param Notafiscal $notaFiscal
     * @throws \Exception
     *
     * @return string
     */
    public static function exec_consulta_grupofiscal(Notafiscal $notaFiscal){
        /**
         * @todo parametros esperados
         * 'F', // Fornecedor.
         * $notafisc['CDFORNECED'],
         * $notafisc['CDFILIAL'],
         * 'E', // Entrada.
         * '01' // Operacao compra.
         */
        try {
            $procedure = new Procedure('CONSULTA_GRUPOFISCAL');
            $procedure->addParam(
                new Param(
                    "P_IDORIGDESTNF",
                    Param::PARAM_INPUT,
                    $notaFiscal->getIdorigdestnf(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_CDORIGDESTNF",
                    Param::PARAM_INPUT,
                    $notaFiscal->getCdforneced(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_CDFILIFATUR",
                    Param::PARAM_INPUT,
                    $notaFiscal->getFilial()->getCdfilial(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_IDENTRSAIDOP",
                    Param::PARAM_INPUT,
                    $notaFiscal->getEntrsaidop()->getIdentrsaidop(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_CDTIPOOPER",
                    Param::PARAM_INPUT,
                    $notaFiscal->getEntrsaidop()->getCdtipooper(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_CDGRUPOFISC",
                    Param::PARAM_OUTPUT,
                    '',
                    Param::PARAM_TYPE_STR,
                    10
                )
            );
            $output = $procedure->call();

        } catch (\Exception $e) {
            throw $e;
        }

        return $output['P_CDGRUPOFISC'];
    }

    public static function procedureGeraVencimento($cdfilial, $nrlanctonf, $idoperacao){
        try {
            $procedure = new Procedure('GERA_DADOS_FINANCEIRO');
            $procedure->addParam(
                new Param(
                    "P_CDFILIAL",
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_NRLANCTONF",
                    Param::PARAM_INPUT,
                    $nrlanctonf,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_CDFILIPROC",
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_IDOPERACAO",
                    Param::PARAM_INPUT,
                    $idoperacao,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "P_RESULTADO",
                    Param::PARAM_OUTPUT,
                    '',
                    Param::PARAM_TYPE_STR,
                    10
                )
            );
            // $procedure = new Procedure('GERA_CONTAPAGAR');
            // $procedure->addParam(
            //     new Param(
            //         "P_CDFILIAL",
            //         Param::PARAM_INPUT,
            //         $cdfilial,
            //         Param::PARAM_TYPE_STR
            //     )
            // );
            // $procedure->addParam(
            //     new Param(
            //         "P_CDFORNECED",
            //         Param::PARAM_INPUT,
            //         $cdforneced,
            //         Param::PARAM_TYPE_STR
            //     )
            // );
            // $procedure->addParam(
            //     new Param(
            //         "P_NRLANCTONF",
            //         Param::PARAM_INPUT,
            //         $nrlanctonf,
            //         Param::PARAM_TYPE_STR
            //     )
            // );
            // $procedure->addParam(
            //     new Param(
            //         "P_IDTIPOLANC",
            //         Param::PARAM_INPUT,
            //         $identrsaidop,
            //         Param::PARAM_TYPE_STR
            //     )
            // );

            $result = $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function procedureDeletaNF(\Fiscal\Entities\Notafiscal $notafiscal, $idcorpitem = null){
        try {
            $procedure = new Procedure('DELETA_NF');
            $procedure->addParam(
                new Param(
                    "p_filial",
                    Param::PARAM_INPUT,
                    $notafiscal->getCdfilial(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "p_lanctonf",
                    Param::PARAM_INPUT,
                    $notafiscal->getNrlanctonf(),
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    "p_idcorpitem",
                    Param::PARAM_INPUT,
                    $idcorpitem,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @param string $atualizaEstoque
     * @return array|bool
     * @throws \Exception
     */
    public static function callProcedureImpostoSaida(Notafiscal $notaFiscal, $atualizaEstoque = 'S')
    {
        try{
            $procedure = new Procedure("IMPOSTO_SAIDA");
            $procedure->addParam(new Param(
                "P_EMPRESA",
                Param::PARAM_INPUT,
                $notaFiscal->getCdempresa(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_INSCESTA",
                Param::PARAM_INPUT,
                $notaFiscal->getCdinscesta(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_TIPOOPER",
                Param::PARAM_INPUT,
                $notaFiscal->getCdtipooper(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CLIENTE",
                Param::PARAM_INPUT,
                $notaFiscal->getCdcliente(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_FILIAL",
                Param::PARAM_INPUT,
                $notaFiscal->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_LANCTONF",
                Param::PARAM_INPUT,
                $notaFiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_INIVALID",
                Param::PARAM_INPUT,
                $notaFiscal->getDtentrsaid()->format('d/m/Y'),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_VRNOTAFISC",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_STR,
                10
            ));
            $procedure->addParam(new Param(
                "P_LANCESTQ",
                Param::PARAM_INPUT,
                $atualizaEstoque,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CDOPERADOR",
                Param::PARAM_INPUT,
                '',
                Param::PARAM_TYPE_STR
            ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function callProcedureGeraVencimento(Notafiscal $notaFiscal, $operacao = "L")
    {
        try {
            $procedure = new Procedure("GERA_DADOS_FINANCEIRO");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $notaFiscal->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $notaFiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_CDFILIPROC",
                Param::PARAM_INPUT,
                \Geral\Util\Ambiente::getCdFilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_IDOPERACAO",
                Param::PARAM_INPUT,
                $operacao,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_RESULTADO",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_STR,
                200
            ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public static function callProcedureImpostoEntradaProduto(Notafiscal $notaFiscal, $grupofiscal, $cdarvprod, $vrproduto, $idinterest)
    {
	
    	try {
    		$procedure = new Procedure("IMPOSTO_ENTRADA_PRODUTO");
    		$procedure->addParam(new Param(
    				"P_EMPRESA",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdempresa(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_INSCESTA",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdinscesta(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_TIPOOPER",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdtipooper(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDFORNECED",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdforneced(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_FILIAL",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdfilial(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_LANCTONF",
    				Param::PARAM_INPUT,
    				$notaFiscal->getNrlanctonf(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_INIVALID",
    				Param::PARAM_INPUT,
    				$notaFiscal->getDtentrsaid()->format(DateUtil::FORMATO_BRASILEIRO),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_ENTRSAID",
    				Param::PARAM_INPUT,
    				$notaFiscal->getIdentrsaidop(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDOPERADOR",
    				Param::PARAM_INPUT,
    				'',
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDGRUPOFISC",
    				Param::PARAM_INPUT,
    				$grupofiscal,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_IDINTEREST",
    				Param::PARAM_INPUT,
    				$idinterest,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDARVPROD",
    				Param::PARAM_INPUT,
    				$cdarvprod,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDGRPFISCSV",
    				Param::PARAM_INPUT,
    				'',
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_IDORIGDESTNF",
    				Param::PARAM_INPUT,
    				$notaFiscal->getIdorigdestnf(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDFILIRECE",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdfilirece(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDFILIEMIT",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdfiliemit(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_CDCLIENTE",
    				Param::PARAM_INPUT,
    				$notaFiscal->getCdcliente(),
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_VRPRODUTO",
    				Param::PARAM_INPUT,
    				$vrproduto,
    				Param::PARAM_TYPE_STR
    		));
				
    		return $procedure->call();
    	} catch (\Exception $e) {
    		throw $e;
    	}
	
    }
    
    public static function callProcedureAtualizaQtrelacNforigem($cdfilial, $nrlanctonf, $nrseqitprod, $vrdiferenca, $idstatus){
    	try {
    		$procedure = new Procedure("ATUALIZA_QTRELAC_ITNFORIGEM");
    		$procedure->addParam(new Param(
    				"P_CDFILIAL",
    				Param::PARAM_INPUT,
    				$cdfilial,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_NRLANCTONF",
    				Param::PARAM_INPUT,
    				$nrlanctonf,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_NRSEQITPROD",
    				Param::PARAM_INPUT,
    				$nrseqitprod,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_VRDIFERENCA",
    				Param::PARAM_INPUT,
    				$vrdiferenca,
    				Param::PARAM_TYPE_STR
    		));
    		$procedure->addParam(new Param(
    				"P_IDSTATUS",
    				Param::PARAM_INPUT,
    				$idstatus,
    				Param::PARAM_TYPE_STR
    		));
    		return $procedure->call();
    	} catch (\Exception $e) {
    		throw $e;
    	}
    }

    public static function callProcedureContabilizaNotaFiscal(Notafiscal $notaFiscal)
    {
        try {
            if($notaFiscal->getIdentrsaidop() == Notafiscal::NOTA_FISCAL_ENTRADA){
                $procedure = new Procedure("CTB_DFE_CONTAB");
                $procedure->addParam(new Param(
                    "P_CDFILIAL",
                    Param::PARAM_INPUT,
                    $notaFiscal->getCdfilial(),
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_NRLANCTONF",
                    Param::PARAM_INPUT,
                    $notaFiscal->getNrlanctonf(),
                    Param::PARAM_TYPE_STR
                ));
                $procedure->addParam(new Param(
                    "P_NRLANCCTB",
                    Param::PARAM_OUTPUT,
                    '',
                    Param::PARAM_TYPE_STR,
                    12
                ));
                return $procedure->call();
            }

            $procedure = new Procedure("CTB_DFS_CONTAB");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $notaFiscal->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $notaFiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCCTB",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_STR,
                12
            ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function callProcedureCancelaDadosFinanceiro(Notafiscal $notaFiscal, $idcanfin = 'S')
    {
        try {
            $procedure = new Procedure("CANCELA_DADOS_FINANCEIRO");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $notaFiscal->getCdfilial(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $notaFiscal->getNrlanctonf(),
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_IDCANFIN",
                Param::PARAM_INPUT,
                $idcanfin,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_RETORNO",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_INT,
                1
            ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function callProcedureAjuste($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag)
    {
        try {
            $procedure = new Procedure("CTB_DFE_AJUSTE");
            $procedure->addParam(new Param(
                "P_CDFILIAL",
                Param::PARAM_INPUT,
                $cdfilial,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCTONF",
                Param::PARAM_INPUT,
                $nrlanctonf,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRSEQVENPAG",
                Param::PARAM_INPUT,
                $nrseqvenpag,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRPARCELAPAG",
                Param::PARAM_INPUT,
                $nrparcelapag,
                Param::PARAM_TYPE_STR
            ));
            $procedure->addParam(new Param(
                "P_NRLANCCTB",
                Param::PARAM_OUTPUT,
                '',
                Param::PARAM_TYPE_STR,
                12
            ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function callProcedureInutilizacao($empresa, $cnpj, $inscricaoEstadual, $inscricaoMunicipal, $serieNfe, $ambiente, $modoOperacional = '1', $geral = 'N')
    {
        try {
            $procedure = new Procedure("INUTILIZACAO_NFE");
            $procedure->addParam(new Param(
                    "P_CDEMPRESA",
                    Param::PARAM_INPUT,
                    $empresa,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_NRINSJURIEST",
                    Param::PARAM_INPUT,
                    $cnpj,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_CDINSCESTA",
                    Param::PARAM_INPUT,
                    $inscricaoEstadual,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_CDINSCMUNI",
                    Param::PARAM_INPUT,
                    $inscricaoMunicipal,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_CDSERIENFE",
                    Param::PARAM_INPUT,
                    $serieNfe,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_IDTPAMBIENTENFE",
                    Param::PARAM_INPUT,
                    $ambiente,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_MODOPERACIONAL",
                    Param::PARAM_INPUT,
                    $modoOperacional,
                    Param::PARAM_TYPE_STR
                ));
            $procedure->addParam(new Param(
                    "P_GERAL",
                    Param::PARAM_INPUT,
                    $geral,
                    Param::PARAM_TYPE_STR
                ));
            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
